﻿<?php
/*
Plugin Name: Fuzzy DateTime in french
Plugin URI: https://wordpress.org/extend/plugins/fuzzy-date-and-time-in-french/
Description: Prints a fuzzy date and time, similar to the KDE clock, in french.
Version: 0.8
Author: mll
Author URI: http://mll2.free.fr/
License: GPL2
*/

/*  Copyright 2007 mll

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License, version 2, as published by the Free Software Foundation. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
*/

/*
This plugin contains code
from the Time of Day plugin by Phu Ly & Dunstan Orchard & Michael Heilemann (http://www.ifelse.co.uk/code/timeofday.php)
and from Fuzzy DateTime plugin by Splee (http://wordpress.org/extend/plugins/splees-fuzzy-datetime/).
*/

//	ini_set('display_errors', true); //for debugging

function filter_post_fuzzy_datetime($post_time) {
	$post_time=fuzzy_datetime($post_time);
    return $post_time;
}

function filter_comment_fuzzy_datetime($comment_time) {
	global $comment;	
	$comment_time = $comment->comment_date;
	$comment_time=fuzzy_datetime($comment_time);
    return $comment_time;
}

function filter_fuzzy_nothing() {
	return "";
}

//(Un)comment at your will. Todo : make this an option.

//add_filter('the_date', 'filter_post_fuzzy_datetime', 999, 4);
//add_filter('the_time', 'filter_post_fuzzy_datetime');
add_filter('get_comment_date', 'filter_comment_fuzzy_datetime');
add_filter('get_comment_time', 'filter_fuzzy_nothing');



// This is the function that needs to replace 'the_time()' in your theme.
function fuzzy_datetime($item_date) {
	//Get the options from the db
	$lm_fdt_options = get_option('lm_fdt_options');
	
	//get the two timestamps
	$nowStampMysql = date('Y-m-d H:i:s', time());
	$nowStamp = strtotime($nowStampMysql);
	$postStampMysql = $item_date; //in mySQL format
	$postStamp = strtotime($postStampMysql);
	$difference = $lm_nowStamp - $lm_postStamp;
	
	
	//Spot the difference...
	$dayDifference = (date('z', $nowStamp) - date('z', $postStamp));
	$monthDifference = (date('n', $nowStamp) - date('n', $postStamp));
	$yearDifference = (date('Y', $nowStamp) - date('Y', $postStamp));
	
	if ($yearDifference > 1) $fuzzyMode = "years";
	elseif ($yearDifference == 1) $fuzzyMode = "year";
	elseif ($monthDifference > 1) $fuzzyMode = "months";
	elseif (($monthDifference == 1) && ($dayDifference >= 14)) $fuzzyMode = "month"; //Added a two week bias before the post was "last month".
	elseif ($dayDifference >= 14) $fuzzyMode = "weeks";
	elseif ($dayDifference >= 7) $fuzzyMode = "week";
	elseif ($dayDifference > 1) $fuzzyMode = "days";
	elseif ($dayDifference == 1) $fuzzyMode = "yesterday";
	elseif ($dayDifference == 0) $fuzzyMode = "today";

	//these are the strings to be replaced
	$rewritecode = array(
		"%minutesSince%",
		"%hoursSince%",
		"%daysSince%",
		"%weeksSince%",
		"%monthsSince%",
		"%yearsSince%",
		"%dayName%",
		"%monthName%",
		"%weekOfMonth%",
		"%fuzzyTimeText%",
		"%fuzzy12Hour%",
		"%fuzzyWeekPeriod%",
		"%fuzzyMonthPeriod%",
		"%MinutesOrHoursSince%",
		"%actualDate%",
		"%actualTime%"
		);

	//get the replacements for the above
//	$rewritereplace = lm_get_fuzzy_replacements(($postStamp + $lm_time_offset), ($nowStamp + $lm_time_offset), $postStampMysql);
	$rewritereplace = lm_get_fuzzy_replacements($postStamp, $nowStamp, $postStampMysql);

	$outString = str_replace($rewritecode, $rewritereplace, $lm_fdt_options[$fuzzyMode]);
	
	//now print the fuzzy datetime.
	return stripslashes($outString);
}


/* lm_get_fuzzy_replacements:
	This function must return an array the same length as the
	$rewriteCode array in fuzzy_datetime()
	
	Parameters
	----------
	$postStamp: This should be the timestamp of the post
	$nowStamp: This should be the timestamp for now()
*/
function lm_get_fuzzy_replacements($postStamp, $nowStamp, $postStampMysql) {
	$difference = ($nowStamp - $postStamp);
	/*	Must return an array with the results in the following order:
	
		"%minutesSince%",
		"%hoursSince%",
		"%daysSince%",
		"%weeksSince%",
		"%monthsSince%",
		"%yearsSince%",
		"%dayName%",
		"%monthName%",
		"%weekOfMonth%",
		"%fuzzyTimeText%",
		"%fuzzy12Hour%",
		"%fuzzyWeekPeriod%",
		"%fuzzyMonthPeriod%",
		"%MinutesOrHoursSince%",
		"%actualDate%",
		"%actualTime%"
	*/
	$returnValue = array();
	// %minutesSince%
	$returnValue[0] = floor($difference/60);
	//%hoursSince%
	$returnValue[1] = floor(($difference/60)/60);
	//%daysSince%
	$returnValue[2] = (date('z', $nowStamp) - date('z', $postStamp)); //floor((($difference/60)/60)/24); 
	//%weeksSince%
	$returnValue[3] = floor(((($difference/60)/60)/24)/7);
	//%monthsSince%
	$returnValue[4] = (date('n', $nowStamp) - date('n', $postStamp));
	//%yearsSince%
	$returnValue[5] = (date('Y', $nowStamp) - date('Y', $postStamp));
		
		if ($returnValue[5] > 0) {
			//deal with the inverted %monthsSince% value if %yearsSince% > 0
			$returnValue[4] = $returnValue[4] * (-1);
			//Also, the daysSince tag will start to go skewiff when calculated in a different year.
			//fix this by being semantically less accurate but numerically more accurate.
			$returnValue[2] = floor(((($difference/60)/60)/24)/7);
		}
	//%dayName%
	$returnValue[6] = mysql2date('l', $postStampMysql);
	//%monthName%
	$returnValue[7] = mysql2date('F', $postStampMysql);
	//%weekOfMonth%
	$returnValue[8] = ceil(date('j', $postStampMysql)/7);
	switch ($returnValue[8]) {
		case 1:
			$returnValue[8] .= "er";
			break;
		case 2:
			$returnValue[8] .= "nd";
			break;
		case 3:
			$returnValue[8] .= "&egrave;me";
			break;
		default:
			$returnValue[8] .= "&egrave;me";
			break;
	}
	//%fuzzyTimeText%
	$returnValue[9] = lm_get_fuzzyTimeText($postStamp);
	//%fuzzy12Hour%
	$returnValue[10] = lm_get_fuzzy12Hour($postStamp);
	// %fuzzyWeekPeriod%
	switch(date('w', $postStamp)) {
		case 0:
			//silly PHP people think sunday is the beginning of the week! *rolls eyes*
			$returnValue[11] = "le week-end";
			break;
		case 1:
		case 2:
			$returnValue[11] = "au d&eacute;but de la semaine";
			break;
		case 3:
		case 4:
			$returnValue[11] = "au milieu de la semaine";
			break;
		case 5:
		case 6:
			$returnValue[11] = "le week-end";
			break;
		default:
			//we don't know what day it was posted, so we'll be super vague here.
			//it was this or an error message; at least this kinda fits in ;)
			$returnValue[11] = "quelque part pendant les 7 derneirs jours";
			break;
	}
	// %fuzzyMonthPeriod%
	// Use the previously calculated monthname
	switch(ceil(date('j', $postStamp)/7)) {
		case 1:
			$returnValue[12] = "au d&eacute;but de " . $returnValue[7];
			break;
		case 2:
		case 3:
			$returnValue[12] = "au milieu de " . $returnValue[7];
			break;
		case 4:
		case 5:
			$returnValue[12] = "&agrave; la fin de " . $returnValue[7];
			break;
		default:
			//again with the nice camouflaged error messages.
			$returnValue[12] = "quelque part pendant " . $returnValue[7];
			break;
	}
	
	//%fuzzyMinutesOrHours%
	//use the previously calculated %hoursSince% and %minutesSince%
	if ($returnValue[1] != 0) {
		//if it's been over an hour:
		if ($returnValue[1] > 1) {
			$returnValue[13] = $returnValue[1] . " heures";
		} else {
			$returnValue[13] = "1 heure";
		}
	} else {
		//it's been less than an hour
		if ($returnValue[0] > 1) {
			//it's more than 1 minute ago
			$returnValue[13] = $returnValue[0] . " minutes";
		} elseif ($returnValue[0] = 1) {
			$returnValue[13] = "1 minute";
		} else {
			//the chances are small, but it's been posted less than a minute ago
			$returnValue[13] = "moins d'une minute";
		}
	}
	//%actualDate%
	$returnValue[14] = mysql2date('D j F Y', $postStamp);
	
	//%actualTime%
	$returnValue[15] = mysql2date('G:i', $postStamp);
	
	return $returnValue;
}

/*
	lm_get_fuzzy12Hour:
	
	This function returns a string containing a
	fuzzy representation of the 12 hour clock.
	
	Parameters
	----------
	$postStamp: This should be the timestamp of the post
*/
function lm_get_fuzzy12Hour($postStamp) {
	//fuzzify the hour
	$fuzHours = array(
		"une",
		"deux",
		"trois",
		"quatre",
		"cinq",
		"six",
		"sept",
		"huit",
		"neuf",
		"dix",
		"onze",
		"douze"
		);
	$returnVal = $fuzHours[(date('g', $postStamp)-1)];
	//fuzzify the minutes
	$postMins = date('i', $postStamp);
	if (($postMins >= 7) && ($postMins <= 26)) {
		//It's around quarter past
		$returnVal = $returnVal . " heures et quart";
	} elseif (($postMins >= 27) && ($postMins <= 37)) {
		//it's around half past
		$returnVal = $returnVal . " heures et demie";
	} elseif (($postMins >= 38) && ($postMins <= 47)) {
		$returnVal = $returnVal . " heures moins le quart";
	}
	$returnVal .= " " . date('a', $postStamp);
	return $returnVal;
}

/*
	lm_get_fuzzyTimeText:
	Returns a string with a fuzzy representation of the time
	period from the $postStamp param.
	
	*based on code from the timeofday plugin by 
	Phu Ly & Dunstan Orchard & Michael Heilemann 
	(http://www.ifelse.co.uk/code/timeofday.php)*
	
	Parameters
	----------
	$postStamp: This should be the timestamp of the post
*/
function lm_get_fuzzyTimeText($postStamp) {
	switch(date('G', $postStamp))
		{
		case 0:
		case 1:
		case 2:
			$returnVal = 'aux petites heures';
			break;
		case 3:
		case 4:
		case 5:
		case 6:
			$returnVal = 'tr&egrave;s t&ocirc;t le matin';
			break;
		case 7:
		case 8:
		case 9:
			$returnVal = 'en d&eacute;but de matin&eacute;e';
			break;
		case 10:
			$returnVal = 'en milieu de matin&eacute;e';
			break;
		case 11:
			$returnVal = 'en fin de matin&eacute;e';
			break;
		case 12:
		case 13:
			$returnVal = 'entre midi et deux';
			break;
		case 14:
			$returnVal = 'en d&eacute;but d\'apr&egrave;-midi';
			break;
		case 15:
		case 16:
			$returnVal = 'en milieu d\'apr&egrave;-midi';
			break;
		case 17:
			$returnVal = 'en fin d\'apr&egrave;-midi';
			break;
		case 18:
		case 19:
			$returnVal = 'en fin de journ&eacute;e';
			break;
		case 20:
		case 21:
			$returnVal = 'en d&eacute;but de soir&eacute;e';
			break;
		case 22:
			$returnVal = 'en fin de soir&eacute;e';
			break;
		case 23:
			$returnVal = 'en fin de soir&eacute;e';
			break;
		default:
			$returnVal = '';
			break;
		}
	return $returnVal;
}

/*
	From here on out we are dealing with the admin GUI.
*/

function lm_fdt_options_page() {
	switch($_POST['action']) {
		case 'lm_fdt_update':
			//The options have just been updated... so update em already!
			$lm_fdt_options["today"] = $_POST[addslashes('today')];
			$lm_fdt_options["yesterday"] = $_POST[addslashes('yesterday')];
			$lm_fdt_options["days"] = $_POST[addslashes('days')];
			$lm_fdt_options["week"] = $_POST[addslashes('week')];
			$lm_fdt_options["weeks"] = $_POST[addslashes('weeks')];
			$lm_fdt_options["month"] = $_POST[addslashes('month')];
			$lm_fdt_options["months"] = $_POST[addslashes('months')];
			$lm_fdt_options["year"] = $_POST[addslashes('year')];
			$lm_fdt_options["years"] = $_POST[addslashes('years')];
			
			update_option('lm_fdt_options', $lm_fdt_options);
			break;
		default:
			//Add the defaults to the db if they're not there already.
			$lm_fdt_defaults["years"] = "il y a %yearsSince% ans, %fuzzyMonthPeriod%";
			$lm_fdt_defaults["year"] = "l'ann&eacute;e derni&egrave;re, %fuzzyMonthPeriod%";
			$lm_fdt_defaults["months"] = "il y a %monthsSince% mois";
			$lm_fdt_defaults["month"] = "il y a un mois";
			$lm_fdt_defaults["weeks"] = "%dayName% d'il y a %weeksSince% semaines";
			$lm_fdt_defaults["week"] = "%dayName% de la semaine derni&egrave;re";
			$lm_fdt_defaults["days"] = "il y a %daysSince% jours, %fuzzyTimeText%";
			$lm_fdt_defaults["yesterday"] = "hier &agrave; %fuzzyTimeText% heures";
			$lm_fdt_defaults["today"] = "il y a %MinutesOrHoursSince%";
			add_option('lm_fdt_options', $lm_fdt_defaults);
			break;
	}
	$lm_fdt_options = get_option('lm_fdt_options');

//the page itself
?>
<div class="wrap">
<form name="lm_fdt_options" method="post" action="<?php echo $_SERVER[PHP_SELF]; ?>?page=fuzzy-datetime-fr.php">
<input type="hidden" name="action" value="lm_fdt_update" />
	<h2>Output Strings</h2>
	<p>Each of these options configure the output depending on the amount of time passed. Enter the string using the replacement variables which are explained at the bottom of the page.</p>
	<table width="100%" cellpadding="5">
		<tr>
			<th valign="top" scope="row" align="right">Today:</th>
			<td>
				<INPUT type="text" name="today" value="<?php echo stripslashes($lm_fdt_options["today"]); ?>" size="70" />
			</td>
		</tr>
		<tr>
			<th valign="top" scope="row" align="right">Yesterday:</th>
			<td>
				<INPUT type="text" name="yesterday" value="<?php echo stripslashes($lm_fdt_options["yesterday"]); ?>" size="70" />
			</td>
		</tr>
		<tr>
			<th valign="top" scope="row" align="right">More than 2 days ago:</th>
			<td>
				<input type="text" name="days" value="<?php echo stripslashes($lm_fdt_options["days"]) ?>" size="70" />
			</td>
		</tr>
		<tr>
			<th valign="top" scope="row" align="right">Last week:</th>
			<td>
				<input type="text" name="week" value="<?php echo stripslashes($lm_fdt_options["week"]) ?>" size="70" />
			</td>
		</tr>
		<tr>
			<th valign="top" scope="row" align="right">More than 1 week ago:</th>
			<td>
				<input type="text" name="weeks" value="<?php echo stripslashes($lm_fdt_options["weeks"]) ?>" size="70" />
			</td>
		</tr>
		<tr>
			<th valign="top" scope="row" align="right">Last month:</th>
			<td>
				<input type="text" name="month" value="<?php echo stripslashes($lm_fdt_options["month"]) ?>" size="70" />
			</td>
		</tr>
		<tr>
			<th valign="top" scope="row" align="right">More than 1 month ago:</th>
			<td>
				<input type="text" name="months" value="<?php echo stripslashes($lm_fdt_options["months"]) ?>" size="70" />
			</td>
		</tr>
		<tr>
			<th valign="top" scope="row" align="right">Last year:</th>
			<td>
				<input type="text" name="year" value="<?php echo stripslashes($lm_fdt_options["year"]) ?>" size="70" />
			</td>
		</tr>
		<tr>
			<th valign="top" scope="row" align="right">More than a year ago:</th>
			<td>
				<input type="text" name="years" value="<?php echo stripslashes($lm_fdt_options["years"]) ?>" size="70" />
			</td>
		</tr>

	</table>
	<p class="submit"><input type="submit" value="Update Options"></p>
	
	<fieldset class="options">
	<legend>Available replacements</legend>
	<table width="100%" cellspacing="2" cellpadding="5">
	<tr>
	<th>String</th><th>Description</th><th>Example usage</th>
	</tr>
	<tr>
	<td>%minutesSince%</td><td>Displays the minutes since the post</td><td>Posted %minutesSince% minutes ago</td>
	</tr>
	<tr>
	<td>%hoursSince%</td><td>Displays the hours since the post</td><td>Posted %hoursSince% hours ago</td>
	</tr>
	<tr>
	<td>%daysSince%</td><td>Displays the days since the post</td><td>Posted %daysSince% days ago</td>
	</tr>
	<tr>
	<td>%weeksSince%</td><td>Displays the weeks since the post</td><td>Posted %weeksSince% weeks ago</td>
	</tr>
	<tr>
	<td>%monthsSince%</td><td>Displays the months since the post</td><td>Posted %monthsSince% months ago</td>
	</tr>
	<tr>
	<td>%yearsSince%</td><td>Displays the years since the post</td><td>Posted %yearsSince% years ago</td>
	</tr>
	<tr>
	<td>%dayName%</td><td>Displays the name of the day when the post was published</td><td>Posted on a %dayName%</td>
	</tr>
	<tr>
	<td>%monthName%</td><td>Displays the name of the month when the post was published</td><td>Posted during %monthName%</td>
	</tr>
	<tr>
	<td>%weekOfMonth%</td><td>Displays the week of the month in which the post was published</td><td>Posted during the %weekOfMonth% week of %monthName%</td>
	</tr>
	<tr>
	<td>%fuzzyTimeText%</td><td>Displays the time, fuzzily</td><td>Posted yesterday, %fuzzyTimeText%</td>
	</tr>
	<tr>
	<td>%fuzzy12Hour%</td><td>Displays the time, fuzzily, in a 12 hour format</td><td>Posted today %fuzzy12Hour%</td>
	</tr>
	<tr>
	<td>%fuzzyWeekPeriod%<td>Displays the general time of the week the post was published</td><td>Posted last week, %fuzzyWeekPeriod%</td>
	</tr>
	<tr>
	<td>%fuzzyMonthPeriod%</td><td>Displays the general time of the month the post was published</td><td>Posted last year, %fuzzyMonthPeriod%</td>
	</tr>
	<tr>
	<td>%MinutesOrHoursSince%</td><td>Diplays the minutes or hours since the post was published, usually used when the post was published that day.</td><td>Posted %MinutesOrHoursSince% ago</td>
	</tr>
	<tr>
	<td>%actualDate%</td><td>Displays the date the post was published</td><td>Posted %actualDate%</td>
	</tr>
	<tr>
	<td>%actualTime%</td><td>Displays the time the post was published</td><td>Posted %actualTime%</td>
	</tr>
	</table>
	</fieldset>
</form>

</div>
<?php
}

/* Add the above pages to the wp-admin GUI */
function lm_fdt_add_pages() {
	add_options_page('Fuzzy DateTime', 'Fuzzy DateTime', 8, basename(__FILE__), 'lm_fdt_options_page');
}
add_action('admin_menu', 'lm_fdt_add_pages');
?>
